// Sample file provided by Nicolas Bonneel (http://www.bonneel.com)
// TreeGenerator sample (http://www.treegenerator.com)

sampler2D p_DiffuseMap : TEXUNIT0;
sampler2D p_ShadowMap : TEXUNIT1;


// Define inputs from application.
struct VertexIn
{
  float4 position   : POSITION;   // Vertex in object-space
  float4 normal     : NORMAL;     // Vertex's Normal
  float2 texCoords  : TEXCOORD0;  // Vertex's Texture Coordinates
};

// Define outputs from vertex shader.
struct Vertex
{
  float4 position        : POSITION;     // Vertex position in screen-space
  float4 texCoordsAndAtm : TEXCOORD0;    // Vertex texture coordinates and Atmospherics values
  float4 normal          : TEXCOORD1;    // Normal in eye-space
  float3 lightVector     : TEXCOORD2;    // Light vector in eye space
  float4 lightPosition   : TEXCOORD3;    // Vertex position in light space 
};

struct Fragment
{
    float4 color  : COLOR0;
};

Vertex mainVP(VertexIn p_In,
             uniform float4x4 p_ModelViewProjection // Model view projection matrix
           , uniform float4 p_LightPosition         // Light position in world-space
           , uniform float4x4 p_Model                 // Model matrix
           , uniform float4x4 p_TextureViewProjection  // Texture view projection matrix
		   , uniform float4 camWorldPos
		   , uniform float4x4 inverseTransposeModel 
		   , uniform float4x4 ModelView

            )
{
    Vertex l_Out;

    // position, normal, and UV coords
    float4 vertexWorldPos = mul(p_Model, p_In.position);
    l_Out.normal = mul(inverseTransposeModel, p_In.normal);
	l_Out.normal.xyz = normalize(l_Out.normal.xyz);
	l_Out.texCoordsAndAtm.xy = p_In.texCoords;    
	l_Out.position =  mul(p_ModelViewProjection, p_In.position);
	
	// lighting and shadowing
    l_Out.lightVector = normalize(p_LightPosition.xyz - vertexWorldPos.xyz);    	    
    l_Out.lightPosition = mul(p_TextureViewProjection, vertexWorldPos);   // light position in shadowmap space 
           
 
    // compute atmospherics per vertex and save registers by saving in UV coordinates    
    l_Out.texCoordsAndAtm.z = exp(-vertexWorldPos.y*0.05) - exp(-camWorldPos.y*0.05);
    l_Out.texCoordsAndAtm.w = exp(-l_Out.position.z*0.002);
    
    return l_Out;
}



Fragment mainFP(Vertex p_In)
{
    Fragment l_Out;

	// base color with diffuse lighting
    float4 DiffuseTex = tex2D( p_DiffuseMap, p_In.texCoordsAndAtm.xy);
    l_Out.color = float4(DiffuseTex.rgb * saturate(dot(p_In.normal.xyz, p_In.lightVector)),  DiffuseTex.a); // note that we use alpha_rejection instead of alpha_blending because of filtering
	 
	// shadowing 
    float2 l_ShadowMapTexCoords = float2(p_In.lightPosition.x, p_In.lightPosition.y) / p_In.lightPosition.w;
    float l_ShadowDistance = tex2D(p_ShadowMap, l_ShadowMapTexCoords).r;
    float visibility = (l_ShadowDistance*1000+0.7) - p_In.lightPosition.z;  
    float l_Lit = lerp(0.4,1, saturate( visibility*1000 ) );  // avoids an "if"
   
    // atmospherics
    l_Out.color.rgb = lerp( l_Lit*l_Out.color.rgb, 1,p_In.texCoordsAndAtm.z); // fog
    l_Out.color.rgb = lerp( float3(0.6,0.6,1), l_Out.color.rgb, p_In.texCoordsAndAtm.w); // aerial perspective
    

    return l_Out;
}
